unit gc;
// Header unit for GC.DLL (Garbage collector)
// Manual translation from GC.H
//
// Version 1.0b1
//
// Delphi source
// Copyright 1997 Vladimir A. Merzlaikov (wanderer@npt.rnd.runnet.ru)
//
// C-source & GC.dll and it sources
// Copyright 1988, 1989 Hans-J. Boehm, Alan J. Demers
// Copyright (c) 1991-1995 by Xerox Corporation.  All rights reserved.
// Copyright 1996 by Silicon Graphics.  All rights reserved.
//
// THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
// OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
//
// Permission is hereby granted to use or copy this program
// for any purpose,  provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//

interface

function GC_malloc( aSize : cardinal ) : Pointer; cdecl;
   // allocate an object of size nbytes.  Unlike malloc, the object is
   // cleared before being returned to the user.  Gc_malloc will
   // invoke the garbage collector when it determines this to be appropriate.
   // GC_malloc may return 0 if it is unable to acquire sufficient
   // space from the operating system.  This is the most probable
   // consequence of running out of space.  Other possible consequences
   // are that a function call will fail due to lack of stack space,
   // or that the collector will fail in other ways because it cannot
   // maintain its internal data structures, or that a crucial system
   // process will fail and take down the machine.  Most of these
   // possibilities are independent of the malloc implementation.
function GC_malloc_atomic( aSize : cardinal ) : Pointer; cdecl;
   // allocate an object of size nbytes that is guaranteed not to contain any
   // pointers.  The returned object is not guaranteed to be cleared.
   // (Can always be replaced by GC_malloc, but results in faster collection
   // times.  The collector will probably run faster if large character
   // arrays, etc. are allocated with GC_malloc_atomic than if they are
   // statically allocated.)
function GC_malloc_uncollectable( aSize : cardinal ) : Pointer; cdecl;
   // allocates an object that is scanned for pointers to collectable objects,
   // but is not itself collectable.
function GC_malloc_atomic_uncollectable( aSize : cardinal ) : Pointer; cdecl;
   // atomic & uncollectable
function GC_malloc_stubborn( aSize : cardinal ) : Pointer; cdecl;
   // promises that no changes to the object will occur after
   // GC_end_stubborn_change has been called on the result of GC_malloc_stubborn
procedure GC_change_stubborn( aPtr : Pointer ); cdecl;
   // stubborn object can change
procedure GC_end_stubborn_change( aPtr : Pointer ); cdecl;
   // stubborn object can't change
function GC_base( aPtr : Pointer ) : Pointer; cdecl;
   // Return a pointer to the base (lowest address) of an object given
   // a pointer to a location within the object.
   // Return 0 if displaced_pointer doesn't point to within a valid object.
function Gc_size( aPtr : Pointer ) : Cardinal; cdecl;
   // Given a pointer to the base of an object, return its size in bytes.
   // The returned size may be slightly larger than what was originally requested.								*/
function GC_realloc( aPtr : Pointer; aSize : cardinal ) : Pointer; cdecl;
   // change the size of object to be new_size.  Returns a pointer to the
   // new object, which may, or may not, be the same as the pointer to
   // the old object.  The new object is taken to be atomic iff the old one
   // was.  If the new object is composite and larger than the original object,
   // then the newly added bytes are cleared (we hope).  This is very likely
   // to allocate a new object, unless MERGE_SIZES is defined in gc_priv.h.
   // Even then, it is likely to recycle the old object only if the object
   // is grown in small additive increments (which, we claim, is generally bad
   // coding practice.)
procedure GC_free( aPtr : Pointer ); cdecl;
   // explicitly deallocate an object returned by GC_malloc or
   // GC_malloc_atomic.  Not necessary, but can be used to minimize
   // collections if performance is critical.  Probably a performance
   // loss for very small objects (<= 8 bytes).
procedure GC_expand_hp( aSize : cardinal ); cdecl;
   // Explicitly increase the heap size.  (This is normally done automatically
   // if a garbage collection failed to GC_reclaim enough memory.  Explicit
   // calls to GC_expand_hp may prevent unnecessarily frequent collections at
   // program startup.)
procedure GC_set_max_heap_size( aSize : cardinal ); cdecl;
   // Limit the heap size to n bytes.  Useful when you're debugging,
   // especially on systems that don't handle running out of memory well.
   // n=0 => unbounded.  This is the default.
procedure GC_clear_roots; cdecl;
   // Clear the set of root segments.  Wizards only!!!
procedure GC_add_roots( aLowPtr,aHighPtrPlus1 : Pointer ); cdecl;
   // Add a root segment.  Wizards only!!!
procedure GC_register_displacement( aDisp : cardinal ); cdecl;
   // Add a displacement to the set of those considered valid by the
   // collector.  GC_register_displacement(n) means that if p was returned
   // by GC_malloc, then (char *)p + n will be considered to be a valid
   // pointer to n.  N must be small and less than the size of p.
   // (All pointers to the interior of objects from the stack are
   // considered valid in any case.  This applies to heap objects and
   // static data.)
   // Preferably, this should be called before any other GC procedures.
   // Calling it later adds to the probability of excess memory
   // retention.
   // This is a no-op if the collector was compiled with recognition of
   // arbitrary interior pointers enabled, which is now the default.

procedure GC_debug_register_displacement( aDisp : Cardinal ); cdecl;
   // The following version should be used if any debugging allocation is
   // being done.
procedure GC_gcollect; cdecl;
   // Explicitly trigger a full, world-stop collection.

type TGC_stop_func = function : boolean; cdecl;
function GC_try_to_collect( aUntil : TGC_stop_func ) : Boolean; cdecl;
   // Trigger a full world-stopped collection.  Abort the collection if
   // and when stop_func returns a nonzero value.  Stop_func will be
   // called frequently, and should be reasonably fast.  This works even
   // if virtual dirty bits, and hence incremental collection is not
   // available for this architecture.  Collections can be aborted faster
   // than normal pause times for incremental collection.  However,
   // aborted collections do no useful work; the next collection needs
   // to start from the beginning.
function GC_get_heap_size : Cardinal; cdecl;
   // Return the number of bytes in the heap.  Excludes collector private
   // data structures.  Includes empty blocks and fragmentation loss.
   // Includes some pages that were allocated but never written.
function GC_get_bytes_since_gc : Cardinal; cdecl;
   // Return the number of bytes allocated since the last collection.
procedure GC_enable_incremental; cdecl;
   // Enables generational and incremental collection.  Useful for large
   // heaps on machines that provide access to page dirty information.
   // Some dirty bit implementations may interfere with debugging
   // (by catching address faults) and place restrictions on heap arguments
   // to system calls (since write faults inside a system call may not be
   // handled well).
function GC_collect_a_little : Boolean; cdecl;
   // Perform some garbage collection work, if appropriate.
   // Return 0 if there is no more work to be done.
   // Typically performs an amount of work corresponding roughly
   // to marking from one page.  May do more work if further
   // progress requires it, e.g. if incremental collection is
   // disabled.  It is reasonable to call this in a wait loop
   // until it returns False

function GC_malloc_ignore_off_page( aSize : Cardinal ) : Pointer; cdecl;
   // identical to GC_malloc, but the client promises to keep a pointer to
   // the somewhere within the first 256 bytes of the object while it is
   // live.  (This pointer should nortmally be declared volatile to prevent
   // interference from compiler optimizations.)  This is the recommended
   // way to allocate anything that is likely to be larger than 100Kbytes
   // or so.  (GC_malloc may result in failure to reclaim such objects.)
function GC_malloc_atomic_ignore_off_page( aSize : Cardinal ) : Pointer; cdecl;
   // Allocate an object of size lb bytes.  The client guarantees that
   // as long as the object is live, it will be referenced by a pointer
   // that points to somewhere within the first 256 bytes of the object.
   // (This should normally be declared volatile to prevent the compiler
   // from invalidating this assertion.)  This routine is only useful
   // if a large array is being allocated.  It reduces the chance of
   // accidentally retaining such an array as a result of scanning an
   // integer that happens to be an address inside the array.  (Actually,
   // it reduces the chance of the allocator not finding space for such
   // an array, since it will try hard to avoid introducing such a false
   // reference.)  On a SunOS 4.X or MS Windows system this is recommended
   // for arrays likely to be larger than 100K or so.  For other systems,
   // or if the collector is not configured to recognize all interior
   // pointers, the threshold is normally much higher.

// Debugging (annotated) allocation.  GC_gcollect will check
// objects allocated in this way for overwrites, etc.
function  GC_debug_malloc( aSize : cardinal; aDesc : PChar; aDescInt : Integer ) : Pointer; cdecl;
function  GC_debug_malloc_atomic( aSize : cardinal; aDesc : PChar; aDescInt : Integer ) : Pointer; cdecl;
function  GC_debug_malloc_uncollectable( aSize : cardinal; aDesc : PChar; aDescInt : Integer ) : Pointer; cdecl;
function  GC_debug_malloc_stubborn( aSize : cardinal; aDesc : PChar; aDescInt : Integer ) : Pointer; cdecl;
procedure GC_debug_free( aPtr : Pointer ); cdecl;
function  GC_debug_realloc( aOld : Pointer; aSize : cardinal; aDesc : PChar; aDescInt : Integer ) : Pointer; cdecl;
procedure GC_debug_change_stubborn( aPtr : Pointer ); cdecl;
procedure GC_debug_end_stubborn_change( aPtr : Pointer ); cdecl;

// Finalization.  Some of these primitives are grossly unsafe.
// The idea is to make them both cheap, and sufficient to build
// a safer layer, closer to PCedar finalization.
// The interface represents my conclusions from a long discussion
// with Alan Demers, Dan Greene, Carl Hauser, Barry Hayes,
// Christian Jacobi, and Russ Atkinson.  It's not perfect, and
// probably nobody else agrees with it.	    Hans-J. Boehm  3/13/92
type TGC_finalization_proc = procedure( aPtr : Pointer; aData : Pointer); cdecl;

procedure GC_register_finalizer( aPtr : Pointer;
                                 aFinalizer : TGC_finalization_proc; const aData;
                                 var aOldFinalizer : TGC_finalization_proc; var aOldData : Pointer
                               ); cdecl;
procedure GC_debug_register_finalizer( aPtr : Pointer;
                                       aFinalizer : TGC_finalization_proc; const aData;
                                       var aOldFinalizer : TGC_finalization_proc; var aOldData : Pointer
                                      ); cdecl;
  // When obj is no longer accessible, invoke (*fn)(obj, cd). If a and b are inaccessible,
  // and a points to b (after disappearing links have been made to disappear),
  // then only a will be finalized.  (If this does not create any new pointers to b,
  // then b will be finalized after the next collection.)  Any finalizable object that
  // is reachable from itself by following one or more pointers will not be
  // finalized (or collected). Thus cycles involving finalizable objects should
  // be avoided, or broken by disappearing links. All but the last finalizer
  // registered for an object is ignored.
  // Finalization may be removed by passing NIL as fn.
  // Finalizers are implicitly unregistered just before they are invoked.
  // The old finalizer and client data are stored in *ofn and *ocd.
  // Fn is never invoked on an accessible object, provided hidden pointers
  // are converted to real pointers only if the allocation lock is held, and
  // such conversions are not performed by finalization routines.
  // If GC_register_finalizer is aborted as a result of a signal, the object may
  // be left with no finalization, even if neither the old nor new finalizer were NIL.
  // Obj should be the nonNULL starting address of an object allocated
  // by GC_malloc or friends.
  // Note that any garbage collectable object referenced by cd will be considered
  // accessible until the finalizer is invoked.

// Another versions of the above follow.  It ignores self-cycles, i.e.
// pointers from a finalizable object to itself.  There is a stylistic argument
// that this is wrong, but it's unavoidable for C++, since the compiler may
// silently introduce these.  It's also benign in that specific case.
procedure GC_register_finalizer_ignore_self( aPtr : Pointer;
                                 aFinalizer : TGC_finalization_proc; const aData;
                                 var aOldFinalizer : TGC_finalization_proc; var aOldData : Pointer
                               ); cdecl;
procedure GC_debug_register_finalizer_ignore_self( aPtr : Pointer;
                                 aFinalizer : TGC_finalization_proc; const aData;
                                 var aOldFinalizer : TGC_finalization_proc; var aOldData : Pointer
                               ); cdecl;

// The following routine may be used to break cycles between finalizable objects,
// thus causing cyclic finalizable objects to be finalized in the correct order.
// Standard use involves calling GC_register_disappearing_link(&p), where p is
// a pointer that is not followed by finalization code, and should not be considered
//  in determining finalization order.
type PPointer = ^Pointer;
function GC_register_disappearing_link( aFieldPtr : PPointer; aObj : Pointer ) : Boolean; cdecl;
  // aFieldPtr *link is cleared when obj first becomes inaccessible.  This
  // can be used to implement weak pointers easily and safely. Typically link will
  // point to a location holding a disguised pointer to obj.  (A pointer inside
  // an "atomic" object is effectively disguised.) In this way soft pointers
  // are broken before any object reachable from them are finalized.  Each link
  // May be registered only once, i.e. with one aObj value. aObj must be a pointer
  // to the first word of an object we allocated.  It is unsafe to explicitly deallocate
  // the object containing link.  Explicitly deallocating obj may or may not cause link
  // to eventually be cleared.
  // Returns TRUE if link was already registered, FALSE otherwise.
function GC_unregister_disappearing_link( aFieldPtr : PPointer ) : Boolean; cdecl;
  // Returns FALSE if link was not actually registered.

// Auxiliary fns to make finalization work correctly with displaced
// pointers introduced by the debugging allocators.
function  GC_make_closure( aFinalizer : TGC_finalization_proc; var aData ) : Pointer; cdecl;
procedure GC_debug_invoke_finalizer( aPtr : Pointer; var aData); cdecl;

// GC_set_warn_proc can be used to redirect or filter warning messages.
// aWarnProc may not be a NIL.
type TGC_warn_proc = procedure( aMsg : PChar; aArg : Cardinal );
function GC_set_warn_proc( aWarnProc : TGC_warn_proc) : TGC_warn_proc; cdecl;
  // Returns old warning procedure.

implementation

function  GC_malloc;                                  external 'GC' name 'GC_malloc';
function  GC_malloc_atomic;                           external 'GC' name 'GC_malloc_atomic';
function  GC_malloc_uncollectable;                    external 'GC' name 'GC_malloc_uncollectable';
function  GC_malloc_atomic_uncollectable;             external 'GC' name 'GC_malloc_atomic_uncollectable';
function  GC_malloc_stubborn;                         external 'GC' name 'GC_malloc_stubborn';
procedure GC_change_stubborn;                         external 'GC' name 'GC_change_stubborn';
procedure GC_end_stubborn_change;                     external 'GC' name 'GC_end_stubborn_change';
function  GC_realloc;                                 external 'GC' name 'GC_realloc';
procedure GC_free;                                    external 'GC' name 'GC_free';
procedure GC_set_max_heap_size;                       external 'GC' name 'GC_set_max_heap_size';
function  GC_base;                                    external 'GC' name 'GC_base';
function  GC_size;                                    external 'GC' name 'GC_size';
function  GC_get_heap_size;                           external 'GC' name 'GC_get_heap_size';
function  GC_get_bytes_since_gc;                      external 'GC' name 'GC_get_bytes_since_gc';
procedure GC_clear_roots;                             external 'GC' name 'GC_clear_roots';
procedure GC_add_roots;                               external 'GC' name 'GC_add_roots';
procedure GC_register_displacement;                   external 'GC' name 'GC_register_displacement';
procedure GC_debug_register_displacement;             external 'GC' name 'GC_debug_register_displacement';
procedure GC_gcollect;                                external 'GC' name 'GC_gcollect';
function  GC_try_to_collect;                          external 'GC' name 'GC_try_to_collect';
function  GC_collect_a_little;                        external 'GC' name 'GC_collect_a_little';
procedure GC_expand_hp;                               external 'GC' name 'GC_expand_hp';
function  GC_malloc_ignore_off_page;                  external 'GC' name 'GC_malloc_ignore_off_page';
function  GC_malloc_atomic_ignore_off_page;           external 'GC' name 'GC_malloc_atomic_ignore_off_page';
procedure GC_enable_incremental;                      external 'GC' name 'GC_enable_incremental';

function  GC_debug_malloc;                            external 'GC' name 'GC_debug_malloc';
function  GC_debug_malloc_atomic;                     external 'GC' name 'GC_debug_malloc_atomic';
function  GC_debug_malloc_uncollectable;              external 'GC' name 'GC_debug_malloc_uncollectable';
function  GC_debug_malloc_stubborn;                   external 'GC' name 'GC_debug_malloc_stubborn';
function  GC_debug_realloc;                           external 'GC' name 'GC_debug_realloc';
procedure GC_debug_free;                              external 'GC' name 'GC_debug_free';
procedure GC_debug_change_stubborn;                   external 'GC' name 'GC_debug_change_stubborn';
procedure GC_debug_end_stubborn_change;               external 'GC' name 'GC_debug_end_stubborn_change';

procedure GC_register_finalizer;                      external 'GC' name 'GC_register_finalizer';
procedure GC_debug_register_finalizer;                external 'GC' name 'GC_debug_register_finalizer';
procedure GC_register_finalizer_ignore_self;          external 'GC' name 'GC_register_finalizer_ignore_self';
procedure GC_debug_register_finalizer_ignore_self;    external 'GC' name 'GC_debug_register_finalizer_ignore_self';
function  GC_register_disappearing_link;              external 'GC' name 'GC_register_disappearing_link';
function  GC_unregister_disappearing_link;            external 'GC' name 'GC_unregister_disappearing_link';
function  GC_make_closure;                            external 'GC' name 'GC_make_closure';
procedure GC_debug_invoke_finalizer;                  external 'GC' name 'GC_debug_invoke_finalizer';
function  GC_set_warn_proc;                           external 'GC' name 'GC_set_warn_proc';

end.
